﻿/** 
 * ---------------------------------------------------------------------------
 * CoolBox
 * ---------------------------------------------------------------------------
 * Rev.1  2017/07/09
 *   変更履歴
 */
using System;
using System.Collections;
using Server.Gumps;
using Server.Multis;
using Server.Network;
using Server.Engines.Plants;
using System.Collections.Generic;
using Server.ContextMenus;
using Server.Prompts;

namespace Server.Items
{
    class CoolBox : Item, ISecurable,ILapisDyable

    {
        private SecureLevel m_Level;
        [CommandProperty(AccessLevel.GameMaster)]
        public SecureLevel Level
        {
            get
            {
                return this.m_Level;
            }
            set
            {
                this.m_Level = value;
            }
        }

        private static bool isDebug = true;
        /** 最大登録数 */
        private static readonly int MAX_COUNT = 9999;
        /** カテゴリ */
        private static readonly String[] category = {
                                                      "浅瀬",
                                                      "深海",
                                                      "ダンジョン",
                                                      "カニ・エビ" };
        private static readonly String[][] CstFishName =
        {
            null,
            new String[] {
                "パンプキンシード","イエローパーチ","パイク",
                "カワマス","ニジマス","ブルーギル",
                "レッドベリーブリーム", "コクチバス","変わり銀魚",
                "ミドリナマズ","ウォールアイ","コカニーマス",
            },
            new String[] {
                "ハドック","ミサキタラ","ブラックシーバス",
                "ターポン","レッドスヌーク","ネズミフエダイ",
                "コビア","マヒマヒ","アンバージャック",
                "シャド","キハダマグロ","カツオ",
                "ブルーフィッシュ","レッドグルーパー","キャプテンスヌーク",
                "ボーンフィッシュ","レッドドラム","ブルーグルーパー",
           },
            new String[] {
                "ダンジョンチャブ","デーモンマス","ラングイバス",
                "カットスロート","グリムシスコ","ドレイクフィッシュ",
                "オークバス","ダークフィッシュ","イワフエダイ",
                "ジゴクマグロ","アワレナパイク","マチブセウオ",
                
            },
            new String[] {
                "ダンジョンクラブ","ブルークラブ","キングクラブ",
                "ロッククラブ","スノークラブ","アップルクラブ",

                "スパイニーロブスター","ロックロブスター","ハンマーロブスター",
                "フレッドロブスター","クラストロブスター","シャベルロブスター",
            },
        };
        /** カテゴリ別魚くん */
        private static readonly Type[][] fish =
        {
            null,
            BaseHighseasFish.ShoreFish,
            BaseHighseasFish.DeepWaterFish,
            BaseHighseasFish.DungeonFish,
            BaseHighseasFish.LobstersAndCrabs
        };
        /** 魚リスト */
        private ArrayList fishList = new ArrayList();
        /** 名前 */
        private String m_coolBoxName = "";

        [CommandProperty(AccessLevel.GameMaster)]
        public string coolBoxName
        {
            get
            {
                return this.m_coolBoxName;
            }
            set
            {
                this.m_coolBoxName = value;                
                this.InvalidateProperties();
            }
        }

        /** 現在表示しているメニューページ番号
         * 0：何も開いていない
         * 1：浅瀬を開いている
         * 2：深海を開いている
         * 3：ダンジョンを開いている
         * 4：エビ・カニを開いている
         */
        private int nowMenuPage = 0;

        /** 現在表示しているメインページ番号 */
        private int nowPage = 0;

        [Constructable]
        public CoolBox()
                : base(0x24D8)
        {
            this.Weight = 1.0;
            this.LootType = LootType.Blessed;
            this.Name = "ラピス漁民のびく";
            this.m_Level = SecureLevel.Owner;
            this.Hue = 0x480;
            if (isDebug)
            {
                for (int i = 0; i < MAX_COUNT; i++)
                {
                }
            }
            // 生成されたタイミングでリストにFishDataを格納
            for(int categoryCnt = 0; categoryCnt < fish.Length; categoryCnt++)
            {
                Type[] wkType = fish[categoryCnt];
                // nullなら次へ
                if (wkType == null) continue;

                // カテゴリに属している魚分ループ
                for(int fishCnt = 0; fishCnt < wkType.Length; fishCnt++)
                {
                    FishData fd = new FishData(categoryCnt, fishCnt,0);
                    // リストに格納
                    fishList.Add(fd);
                }
            }
        }
        public CoolBox(Serial serial)
            : base(serial)
        {
        }
        public override void GetContextMenuEntries(Mobile from, List<ContextMenuEntry> list)
        {
            base.GetContextMenuEntries(from, list);

            if (from.CheckAlive() && this.IsChildOf(from.Backpack))
                list.Add(new NameCoolBoxEntry(from, this));

            SetSecureLevelEntry.AddTo(from, this, list);
        }
        /**
         * 登録されている魚の総数を返します。
         */
        public int getTotalCount()
        {
            int count = 0;
            for(int i = 0; i < fishList.Count; i++)
            {
                FishData fd = (FishData)fishList[i];
                count += fd.getCount();
            }
            return count;
        }
        /**
         * 指定したタイプの魚の総数を返します。
         */
        public int getFishCount(BaseHighseasFish pType)
        {
            int count = 0;
            for (int i = 0; i < fishList.Count; i++)
            {
                FishData fd = (FishData)fishList[i];
                // カテゴリと魚番号から魚クラスを取得
                try
                {
                    Type type = fish[fd.getPageNum()][fd.getTypeNum()];
                    if (type == pType.GetType())
                    {
                        count = fd.getCount();
                        break;
                    }
                }
                catch
                {
                    count = 0;
                    break;
                }
            }
            return count;
        }
        /**
         * 指定したタイプの魚のFishDataを返します。
         */
        public FishData getFishData(BaseHighseasFish pType)
        {
            FishData fd = null;
            for (int i = 0; i < fishList.Count; i++)
            {
                fd = (FishData)fishList[i];
                // カテゴリと魚番号から魚クラスを取得
                try
                {
                    Type type = fish[fd.getPageNum()][fd.getTypeNum()];
                    if(type == pType.GetType())
                    {
                        break;
                    }
                }
                catch
                {
                    fd = null;
                    break;
                }
            }
            return fd;
        }

        public override void OnDoubleClick(Mobile from)
        {
            if (from.FindGump(typeof(CoolBox.InternalGump)) == null)
                from.SendGump(new InternalGump(this, 0));
        }

        public override bool OnDragDrop(Mobile from, Item dropped)
        {

            // ドロップされたアイテムが魚ではない
            if (!(dropped is BaseHighseasFish))
            {
                from.SendMessage("そういうイジワルはしちゃあかんです！！");
                return false;
            }
            // Rev.1(S)
            // レアチェック
            if(dropped is RareFish)
            {
                from.SendMessage("それは大事に取っておくんだ！");
                return false;
            }
            // Rev.1(E)
            
            BaseHighseasFish dFish = null;
            // チェック
            int stack = 0;
            if (dropped is BaseHighseasFish)
            {
                // 登録数MAX(以上はありえないけど念のため)
                if (getTotalCount() >= MAX_COUNT)
                {
                    from.SendMessage("これ以上追加できません。");
                    return false;
                }
                // 魚はスタックするのでスタック数を考慮したMAX数チェック
                stack = dropped.Amount;
                // 現在の登録数＋スタック数が登録数MAX以上
                if((getTotalCount() + stack) > MAX_COUNT)
                {
                    from.SendMessage("これ以上追加できません。");
                    return false;
                }
                dFish = (BaseHighseasFish)dropped;
            }
            // ありえないだろうけど一応
            if(fish == null)
            {
                return false;
            }
            // スタック数分追加する
            FishData fd = getFishData(dFish);
            fd.setCount(fd.getCount() + stack);

            FishData fd2 = getFishData(dFish);

            from.SendMessage("魚を追加しました！");
            dropped.Delete();
            // プロパティを更新
            this.InvalidateProperties();
            // バインダーのガンプを表示していたら閉じる
            from.CloseGump(typeof(InternalGump));
            // バインダーを開く
            from.SendGump(new InternalGump(this, 1));
            return true;
        }

         /**
         * 現在のメニューページを返します。
         */
        public int getNowMenuPage()
        {
            return nowMenuPage;
        }
        /**
         * 指定したのメニューページへ進みます
         */
        public void gotoMenuPage(int menuPage)
        {
            this.nowMenuPage = menuPage;
        }
        /**
         * 現在のメインページを返します。
         */
        public int getNowPage()
        {
            return nowPage;
        }
        /**
         * 指定したのメインページへ進みます
         */
        public void gotoPage(int page)
        {
            this.nowPage = page;
        }
        /**
         * 魚を復元します。
         */
        public void releseFish(Mobile mob,int typeNum)
        {
            if (CoolBox.isDebug)
            {
            }
            // 重量判定
            if ((mob.MaxWeight - mob.TotalWeight) < 50)
            {
                mob.SendMessage("バックパックの重量に余裕がありません。");
                return;
            }
            // 所持数判定
            if (mob.Backpack.TotalItems > 124)
            {
                mob.SendMessage("バックパックの空きに余裕がありません。");
                return;
            }
            // FishData取得
            Type type = fish[this.getNowMenuPage()][typeNum];
            Item item = Activator.CreateInstance(type) as Item;
            FishData fd = this.getFishData((BaseHighseasFish)item);

            if(fd == null || fd.getCount() <= 0)
            {
                mob.SendMessage("該当の魚がありません。");
                return;
            }
            fd.setCount(fd.getCount() - 1);
            // 存在する場合はバックパックに復元
            mob.SendMessage("魚を取り出しました。");
            // プロパティを更新
            this.InvalidateProperties();
            Item backpackFish = mob.Backpack.FindItemByType(type);
            if (backpackFish != null)
            {
                backpackFish.Amount += 1;
            }
            else
            {
                mob.Backpack.DropItem(item);
            }
        }

        /**
         * メニューリスト(日本語）を返します
         */
        public String[] getMenuFishListName(int category)
        {
            return CstFishName[category];
        }

        public override void GetProperties(ObjectPropertyList list)
        {
            base.GetProperties(list);
            list.Add(1062344, this.getTotalCount().ToString() + "/" + MAX_COUNT);
            if (this.m_coolBoxName != null && this.m_coolBoxName.Length > 0)
                list.Add("名前 : " + this.m_coolBoxName); // Book Name: ~1_val~
        }

        private class InternalGump : Gump
        {
            private int mainPageX = 260;
            private int nKind = 0;

            // リストから割り出した必要ページ数
            private int page = 0;

            // 1ページあたりの最大表示数
            private static readonly int PAGE_MAX_DISP = 10;
            // 半ページあたりの最大表示数
            private static readonly int HALF_PAGE_MAX_DISP = 5;
            // 最大ページ数
            private static readonly int PAGE_MAX = 3;
            // ガンプの大きさ
            private static readonly int DUMP_HEIGHT = 550;


            private CoolBox coolBox;
            public InternalGump(CoolBox box, int nKind)
                : base(150, 50)
            {
                this.coolBox = box;
                this.AddPage(0);
                // 背景
                this.AddImageTiled(20, 18, 560, DUMP_HEIGHT, 0x0A8E);
                // 左上
                this.AddImage(0, 0, 0x0A3C);                   // width20 height18
                // 上
                this.AddImageTiled(20, 0, 560, 18, 0x0A3D);
                // 右上
                this.AddImage(560, 0, 0x0A3E);        // width26 height26
                // 左
                this.AddImageTiled(0, 18, 20, DUMP_HEIGHT - 18, 0x0A3F);
                // 右
                this.AddImageTiled(560, 18, 20, DUMP_HEIGHT - 18, 0x0A41);


                // タイトルから少し上に横線
                this.AddImageTiled(4, 20, 570, 5, 0x0A3D);
                // タイトルから少し下に横線
                this.AddImageTiled(4, 140, 570, 5, 0x0A3D);

                // 下
                this.AddImageTiled(20, DUMP_HEIGHT, 560 - 20, 18, 0x0A43);
                // 左下
                this.AddImage(0, DUMP_HEIGHT, 0x0A42);
                // 右下
                this.AddImage(560, DUMP_HEIGHT, 0x0A44);

                // メニュー区切り
                this.AddImageTiled(265, 135 + 11, 20, DUMP_HEIGHT - 135, 0x0A3F);

                this.AddHtml(20, 30, 180, 20, "<BASEFONT COLOR =#4169E1><CENTER><BIG>ラピス漁民のびく", false, false);
                this.AddItem(80, 80, 0x3B02);
                this.AddItem(170, 70, 0x3B0A);
                this.AddItem(120, 80, 0x0CA7);
                int buttonheight = 22;
                int categoryoffset = 4;
                this.AddButton(230, 30, 0x0FAE, 0x0FB0, 101, GumpButtonType.Reply, 0);
                this.AddHtml(230 + 37, 31, 200, 20, "<BASEFONT COLOR =#FFFFFF>浅瀬", false, false);

                this.AddButton(230, 30 + buttonheight + categoryoffset, 0x0FAE, 0x0FB0, 102, GumpButtonType.Reply, 0);
                this.AddHtml(230 + 37, 31 + buttonheight + categoryoffset, 200, 20, "<BASEFONT COLOR =#FFFFFF>深海", false, false);

                this.AddButton(230, 30 + (buttonheight*2) + (categoryoffset*2), 0x0FAE, 0x0FB0, 103, GumpButtonType.Reply, 0);
                this.AddHtml(230 + 37, 31 + (buttonheight * 2) + (categoryoffset * 2), 200, 20, "<BASEFONT COLOR =#FFFFFF>ダンジョン", false, false);

                this.AddButton(230, 30 + (buttonheight * 3) + (categoryoffset * 3), 0x0FAE, 0x0FB0, 104, GumpButtonType.Reply, 0);
                this.AddHtml(230 + 37, 31 + (buttonheight * 3) + (categoryoffset * 3), 200, 20, "<BASEFONT COLOR =#FFFFFF>カニ・エビ", false, false);

                // メニューの描画
                if (this.coolBox.getNowMenuPage() != 0)
                {
                    for (int i = 0; i < this.coolBox.getMenuFishListName(this.coolBox.getNowMenuPage()).Length; i++)
                    {
                        Type type = CoolBox.fish[this.coolBox.getNowMenuPage()][i];
                        Item item = Activator.CreateInstance(type) as Item;
                        int count = this.coolBox.getFishCount((BaseHighseasFish)item);

                        this.AddButton(30, 140 + (20 * (i + 1)), 0x0837, 0x0838, (i + 1), GumpButtonType.Reply, 0);

                        String text = this.coolBox.getMenuFishListName(this.coolBox.getNowMenuPage())[i] + "(" + count + ")";
                        this.AddHtml(55, 138 + (20 * (i + 1)), 200, 20, "<BASEFONT COLOR =#FFFFFF>" + text, false, false);
                    }
                }

            }

            public override void OnResponse(NetState sender, RelayInfo info)
            {
                int button = info.ButtonID;
                Mobile mob = sender.Mobile;

                switch (button)
                {
                    case 101:
                        this.coolBox.gotoMenuPage(1);
                        this.coolBox.gotoPage(0);
                        mob.CloseGump(typeof(InternalGump));
                        mob.SendGump(new InternalGump(this.coolBox, 1));
                        break;
                    case 102:
                        this.coolBox.gotoMenuPage(2);
                        this.coolBox.gotoPage(0);
                        mob.CloseGump(typeof(InternalGump));
                        mob.SendGump(new InternalGump(this.coolBox, 1));
                        break;
                    case 103:
                        this.coolBox.gotoMenuPage(3);
                        this.coolBox.gotoPage(0);
                        mob.CloseGump(typeof(InternalGump));
                        mob.SendGump(new InternalGump(this.coolBox, 1));
                        break;
                    case 104:
                        this.coolBox.gotoMenuPage(4);
                        this.coolBox.gotoPage(0);
                        mob.CloseGump(typeof(InternalGump));
                        mob.SendGump(new InternalGump(this.coolBox, 1));
                        break;
                    default:
                        if (button > 0 && button < 19){
                            coolBox.releseFish(mob, button - 1);
                            mob.CloseGump(typeof(InternalGump));
                            mob.SendGump(new InternalGump(this.coolBox, 1));
                        }
                        else
                        {
                            mob.CloseGump(typeof(InternalGump));
                        }
                        break;
                }
            }

        }

        /** デバッグメッセージ出力*/
        public static void dMsg(String msg)
        {
            if (isDebug)
            {
                Console.WriteLine(msg);
            }
        }


        private class NameCoolBoxEntry : ContextMenuEntry
        {
            private readonly Mobile m_From;
            private readonly CoolBox m_CoolBox;
            public NameCoolBoxEntry(Mobile from, CoolBox coolBox)
                : base(6216)
            {
                this.m_From = from;
                this.m_CoolBox = coolBox;
            }

            public override void OnClick()
            {
                if (this.m_From.CheckAlive() && this.m_CoolBox.IsChildOf(this.m_From.Backpack))
                {
                    this.m_From.Prompt = new NameCoolBoxPrompt(this.m_CoolBox);
                    this.m_From.SendMessage("新しい名前を付けてください。"); // Type in the new name of the book:
                }
            }
        }
        private class NameCoolBoxPrompt : Prompt
        {
            private readonly CoolBox m_CoolBox;
            public NameCoolBoxPrompt(CoolBox coolBox)
            {
                this.m_CoolBox = coolBox;
            }

            public override void OnResponse(Mobile from, string text)
            {
                if (text.Length > 40)
                    text = text.Substring(0, 40);

                if (from.CheckAlive() && this.m_CoolBox.IsChildOf(from.Backpack))
                {
                    this.m_CoolBox.coolBoxName = Utility.FixHtml(text.Trim());

                    from.SendMessage("名前を変更しました。"); // The bulk order book's name has been changed.
                }
            }

            public override void OnCancel(Mobile from)
            {
            }
        }

        public class FishData
        {
            private readonly int pageNum;
            private readonly int typeNum;
            private int count;

            public FishData(int pNum, int tNum, int count)
            {
                this.pageNum = pNum;
                this.typeNum = tNum;
                this.count = count;
            }

            public FishData(GenericReader reader)
            {
                int version  = reader.ReadInt();
                this.pageNum = reader.ReadInt();
                this.typeNum = reader.ReadInt();
                this.count   = reader.ReadInt();
            }

            public void setCount(int cnt)
            {
                this.count = cnt;
            }

            public int getPageNum()
            {
                return this.pageNum;
            }
            public int getTypeNum()
            {
                return this.typeNum;
            }
            public int getCount()
            {
                return this.count;
            }

            public void Serialize(GenericWriter writer)
            {
                writer.Write(0); // version
                writer.Write(pageNum);
                writer.Write(typeNum);
                writer.Write(count);
            }
        }

        public override void Serialize(GenericWriter writer)
        {
            base.Serialize(writer);

            writer.Write((int)0); // version
            writer.Write((int)m_Level); // セキュアレベル
            writer.Write((int)this.fishList.Count); // 総数
            for (int i = 0; i < this.fishList.Count; ++i)
            {
                object obj = this.fishList[i];
                ((FishData)obj).Serialize(writer);
            }
            writer.Write(this.coolBoxName); // 名前
            writer.Write(this.Hue); // 色
        }


        public override void Deserialize(GenericReader reader)
        {
            base.Deserialize(reader);
            int version = reader.ReadInt();
            this.m_Level = (SecureLevel)reader.ReadInt();

            int count = reader.ReadInt();
            this.fishList = new ArrayList();
            for (int i = 0; i < count; i++)
            {
                FishData pd = new FishData(reader);
                this.fishList.Add(pd);
            }
            this.coolBoxName = reader.ReadString();
            this.Hue = reader.ReadInt();
        }
    }
}
